# -*- encoding: utf-8 -*-
# stub: rake-toolkit_program 0.1.2 ruby lib

Gem::Specification.new do |s|
  s.name = "rake-toolkit_program".freeze
  s.version = "0.1.2"

  s.required_rubygems_version = Gem::Requirement.new(">= 0".freeze) if s.respond_to? :required_rubygems_version=
  s.require_paths = ["lib".freeze]
  s.authors = ["Richard Weeks".freeze]
  s.bindir = "exe".freeze
  s.date = "2020-04-13"
  s.description = "# Rake::ToolkitProgram\n\nCreate toolkit programs easily with `Rake` and `OptionParser` syntax.  Bash completions and usage help are baked in.\n\n## Installation\n\nAdd this line to your application's Gemfile:\n\n```ruby\ngem 'rake-toolkit_program'\n```\n\nAnd then execute:\n\n    $ bundle\n\nOr install it yourself as:\n\n    $ gem install rake-toolkit_program\n\n## Quickstart\n\n* Shebang it up (in a file named `awesome_tool.rb`)\n  ```ruby\n  #!/usr/bin/env ruby\n  ```\n* Require the library\n  ```ruby\n  require 'rake/toolkit_program'\n  ```\n* Make your life easier\n  ```ruby\n  Program = Rake::ToolkitProgram\n  ```\n* Define your command tasks\n  ```ruby\n  Program.command_tasks do\n    desc \"Build it\"\n    task 'build' do\n      # Ruby code here\n    end\n    \n    desc \"Test it\"\n    task 'test' => ['build'] do\n      # Rake syntax \u2191\u2191\u2191\u2191\u2191\u2191\u2191 for dependencies\n      # Ruby code here\n    end\n  end\n  ```\n  You can use `Program.args` in your tasks to access the other arguments on the command line.  For argument parsing integrated into the help provided by the program, see the use of `Rake::Task(Rake::ToolkitProgram::TaskExt)#parse_args` below.\n* Wire the mainline\n  ```ruby\n  Program.run(on_error: :exit_program!) if $0 == __FILE__\n  ```\n* In the shell, prepare to run the program (UNIX/Linux systems only)\n  ```console\n  $ chmod +x awesome_tool.rb\n  $ ./awesome_tool.rb --install-completions\n  Completions installed in /home/rtweeks/.bashrc\n  Source /home/rtweeks/.bash-complete/awesome_tool.rb-completions for immediate availability.\n  $ source /home/rtweeks/.bash-complete/awesome_tool.rb-completions\n  ```\n* Ask for help\n  ```console\n  $ ./awesome_tool.rb help\n  \n  *** ./awesome_tool.rb Toolkit Program ***\n  \n      .\n      .\n      .\n  ```\n\n## Usage\n\nLet's look at a short sample toolkit program -- put this in `awesome.rb`:\n\n```ruby\n#!/usr/bin/env ruby\nrequire 'rake/toolkit_program'\nrequire 'ostruct'\n\nToolkitProgram = Rake::ToolkitProgram\nToolkitProgram.title = \"My Awesome Toolkit of Awesome\"\n\nToolkitProgram.command_tasks do\n  desc <<-END_DESC.dedent\n    Fooing myself\n  \n    I'm not sure what I'm doing, but I'm definitely fooing!\n  END_DESC\n  task :foo do\n    a = ToolkitProgram.args\n    puts \"I'm fooed\#{' on a ' if a.implement}\#{a.implement}\"\n  end.parse_args(into: OpenStruct.new) do |parser, args|\n    parser.no_positional_args!\n    parser.on('-i', '--implement IMPLEMENT', 'An implement on which to be fooed') do |val|\n      args.implement = val\n    end\n  end\nend\n\nif __FILE__ == $0\n  ToolkitProgram.run(on_error: :exit_program!)\nend\n```\n\nMake sure to `chmod +x awesome.rb`!\n\nWhat does this support?\n\n    $ ./awesome.rb foo\n    I'm fooed\n    $ ./awesome.rb --help\n    \n    *** My Awesome Toolkit of Awesome ***\n\n    Usage: ./awesome.rb COMMAND [OPTION ...]\n\n    Avaliable options vary depending on the command given. For details\n    of a particular command, use:\n\n        ./awesome.rb help COMMAND\n\n    Commands:\n         foo   Fooing myself\n        help   Show a list of commands or details of one command\n\n    Use help COMMAND to get more help on a specific command.\n    \n    $ ./awesome.rb help foo\n    \n    *** My Awesome Toolkit of Awesome ***\n\n    Usage: ./awesome.rb foo [OPTION ...]\n\n    Fooing myself\n\n    I'm not sure what I'm doing, but I'm definitely fooing!\n\n    Options:\n        -i, --implement IMPLEMENT        An implement on which to be fooed\n    \n    $ ./awesome.rb --install-completions\n    Completions installed in /home/rtweeks/.bashrc\n    Source /home/rtweeks/.bash-complete/awesome.rb-completions for immediate availability.\n    $ source /home/rtweeks/.bash-complete/awesome.rb-completions\n    $ ./awesome.rb <tab><tab>\n    foo   help\n    $ ./awesome.rb f<tab>\n    \u21B3 ./awesome.rb foo\n    $ ./awesome.rb foo <tab>\n    \u21B3 ./awesome.rb foo --\n    $ ./awesome.rb foo --<tab><tab>\n    --help       --implement\n    $ ./awesome.rb foo --i<tab>\n    \u21B3 ./awesome.rb foo --implement\n    $ ./awesome.rb foo --implement <tab><tab>\n    --help      awesome.rb\n    $ ./awesome.rb foo --implement spoon\n    I'm fooed on a spoon\n\n### Defining Toolkit Commands\n\nJust define tasks in the block of `Rake::ToolkitProgram.command_tasks` with `task` (i.e. `Rake::DSL#task`).  If `desc` is used to provide a description, the task will become visible in help and completions.\n\nWhen a command task is initially defined, positional arguments to the command are available as an `Array` through `Rake::ToolkitProgram.args`.\n\n### Option Parsing\n\nThis gem extends `Rake::Task` with a `#parse_args` method that creates a `Rake::ToolkitProgram::CommandOptionParser` (derived from the standard library's `OptionParser`) and an argument accumulator and `yield`s them to its block.\n* The arguments accumulated through the `Rake::ToolkitProgram::CommandOptionParser` are available to the task in `Rake::ToolkitProgram.args`, replacing the normal `Array` of positional arguments.\n* Use the `into:` keyword of `#parse_args` to provide a custom argument accumulator object for the associated command.  The default argument accumulator constructor can be defined with `Rake::ToolkitProgram.default_parsed_args`.  Without either of these, the default accumulator is a `Hash`.\n* Options defined using `OptionParser#on` (or any of the variants) will print in the help for the associated command.\n\n### Positional Arguments\n\nAccessing positional arguments given after the command name depends on whether or not `Rake::Task(Rake::ToolkitProgram::TaskExt)#parse_args` has been called on the command task.  If this method is not called, positional arguments will be an `Array` accessible through `Rake::ToolkitProgram.args`.\n\nWhen `Rake::Task(Rake::ToolkitProgram::TaskExt)#parse_args` is used:\n* `Rake::ToolkitProgram::CommandOptionParser#capture_positionals` can be used to define how positional arguments are accumulated.\n  * If the argument accumulator is a `Hash`, the default (without calling this method) is to assign the `Array` of positional arguments to the `nil` key of the `Hash`.\n  * For other types of accumulators, the positional arguments are only accessible if `Rake::ToolkitProgram::CommandOptionParser#capture_positionals` is used to define how they are captured.\n  * If a block is given to this method, the block of the method will receive the `Array` of positional arguments.  If it is passed an argument value, that value is used as the key under which to store the positional arguments if the argument accumulator is a `Hash`.\n* `Rake::ToolkitProgram::CommandOptionParser#expect_positional_cardinality` can be used to set a rule for the count of positional arguments.  This will affect the _usage_ presented in the help for the associated command.\n* `Rake::ToolkitProgram::CommandOptionParser#map_positional_args` may be used to transform (or otherwise process) positional arguments one at a time and in the context of options and/or arguments appearing earlier on the command line.\n\n### Convenience Methods\n\n* `Rake::Task(Rake::ToolkitProgram::TaskExt)#prohibit_args` is a quick way, for commands that accept no options or positional arguments, to declare this so the help and bash completions reflect this.  It is equivalent to using `#parse_args` and telling the parser `parser.expect_positional_cardinality(0)`.\n\n* `Rake::ToolkitProgram::CommandOptionParser#no_positional_args!` is a shortcut for calling `#expect_positional_cardinality(0)` on the same object.\n\n* `Rake::Task(Rake::ToolkitProgram::TaskExt)#invalid_args!` and `Rake::ToolkitProgram::CommandOptionParser#invalid_args!` are convenient ways to raise `Rake::ToolkitProgram::InvalidCommandLine` with a message.\n\n## OptionParser in Rubies Before and After v2.4\n\nThe `OptionParser` class was extended in Ruby 2.4 to simplify capturing options into a `Hash` or other container implementing `#[]=` in a similar way.  This gem supports that, but it means that behavior varies somewhat between the pre-2.4 era and the 2.4+ era.  To have consistent behavior across that version change, the recommendation is to use a `Struct`, `OpenStruct`, or custom class to hold program options rather than `Hash`.\n\n## Development\n\nAfter checking out the repo, run `bin/setup` to install dependencies. You can also run `bin/console` for an interactive prompt that will allow you to experiment.\n\nTo install this gem onto your local machine, run `bundle exec rake install`. To release a new version, update the version number in `version.rb`, and then run `bundle exec rake release`, which will create a git tag for the version, push git commits and tags, and push the `.gem` file to [rubygems.org](https://rubygems.org).\n\nTo run the tests, use `rake`, `rake test`, or `rspec spec`.  Tests can only be run on systems that support `Kernel#fork`, as this is used to present a pristine and isolated environment for setting up the tool.  If run using Ruby 2.3 or earlier, some tests will be pending because functionality expects Ruby 2.4's `OptionParser`.\n\n## Contributing\n\nBug reports and pull requests are welcome on GitHub at https://github.com/PayTrace/rake-toolkit_program.  For further details on contributing, see [CONTRIBUTING.md](./CONTRIBUTING.md).\n\n".freeze
  s.email = ["rtweeks21@gmail.com".freeze]
  s.homepage = "https://github.com/PayTrace/rake-toolkit_program".freeze
  s.licenses = ["Apache-2.0".freeze]
  s.rubygems_version = "3.2.22".freeze
  s.summary = "Build powerful CLI toolkits with simple code".freeze

  s.installed_by_version = "3.2.22" if s.respond_to? :installed_by_version

  if s.respond_to? :specification_version then
    s.specification_version = 4
  end

  if s.respond_to? :add_runtime_dependency then
    s.add_runtime_dependency(%q<rake>.freeze, [">= 10.0"])
    s.add_runtime_dependency(%q<colorize>.freeze, ["~> 0.8"])
    s.add_runtime_dependency(%q<dedent>.freeze, ["~> 1.0"])
    s.add_development_dependency(%q<bundler>.freeze, [">= 1.13", "< 3"])
    s.add_development_dependency(%q<rspec>.freeze, ["~> 3.9"])
    s.add_development_dependency(%q<pry>.freeze, [">= 0"])
    s.add_development_dependency(%q<pry-coolline>.freeze, [">= 0"])
  else
    s.add_dependency(%q<rake>.freeze, [">= 10.0"])
    s.add_dependency(%q<colorize>.freeze, ["~> 0.8"])
    s.add_dependency(%q<dedent>.freeze, ["~> 1.0"])
    s.add_dependency(%q<bundler>.freeze, [">= 1.13", "< 3"])
    s.add_dependency(%q<rspec>.freeze, ["~> 3.9"])
    s.add_dependency(%q<pry>.freeze, [">= 0"])
    s.add_dependency(%q<pry-coolline>.freeze, [">= 0"])
  end
end
